/*
  Smoothing
 
 Legge 10 valori dal sensore e ritorna il valore medio tra questi.

 Schema: https://learn.adafruit.com/assets/460

 
 
 */

// These constants won't change:
const int sensorPin = A0;    // pin that the sensor is attached to
const int ledPin = 11;        // pin that the LED is attached to

// variables:
int sensorValue = 0;         // the sensor value
int sensorMin = 1023;        // minimum sensor value
int sensorMax = 0;           // maximum sensor value


void setup() {
  // turn on LED to signal the start of the calibration period:
  pinMode(13, OUTPUT);
  pinMode(ledPin, OUTPUT);
  digitalWrite(13, HIGH);

  // calibrate during the first five seconds 
  while (millis() < 5000) {
    sensorValue = analogRead(sensorPin);

    // record the maximum sensor value
    if (sensorValue > sensorMax) {
      sensorMax = sensorValue;
    }

    // record the minimum sensor value
    if (sensorValue < sensorMin) {
      sensorMin = sensorValue;
    }
  }

  // signal the end of the calibration period
  digitalWrite(13, LOW);
}

void loop() {
  // read the sensor:
  sensorValue = smoothRead(sensorPin);

  // apply the calibration to the sensor reading
  sensorValue = map(sensorValue, sensorMin, sensorMax, 0, 255);

  // in case the sensor value is outside the range seen during calibration
  sensorValue = constrain(sensorValue, 0, 255);

  // fade the LED using the calibrated value:
  analogWrite(ledPin, sensorValue);
}
// Funzioni

int smoothRead(int sensorPin) {
//  Legge 10 valori dal sensore e ritorna il valore medio tra questi.
  int total = 0;
  for (int i = 0; i < 10; i++) { 
    total = total + analogRead(sensorPin);
    delay(2); // Pausa per assestare il sensore
  }
  return(total / 10); // Valore medio
  /* Nota: i microcontroller non sono bravi a fare le divisioni,
     sarebbe piu' efficiente ottenere 8 valori e fare uno shift dx. di 3 bit
     total = total >> 3 ;
Vedi: https://www.arduino.cc/en/Reference/Bitshift
   */
}


